/******************************************************************/
/*                                                                */
/* Copyright (C) 1994 - 2006, IBM, Ltd.                           */
/* All Rights Reserved                                            */
/*                                                                */
/*----------------------------------------------------------------*/
/*                                                                */
/* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE                    */
/* The copyright notice above does not evidence any               */
/* actual or intended publication of such source code.            */
/*                                                                */
/* RESTRICTED RIGHTS:                                             */
/*                                                                */
/* This file may have been supplied under a license.              */
/* It may be used, disclosed, and/or copied only as permitted     */
/* under such license agreement. Any copy must contain the        */
/* above copyright notice and this restricted rights notice.      */
/* Use, copying, and/or disclosure of the file is strictly        */
/* prohibited unless otherwise provided in the license agreement. */
/*                                                                */
/******************************************************************/

/**
********************************************************************************
*
* PropertyValue()
*
* Return the value of a property as a string.
*
* @param propName Name of the property to return the value of.
*
* @return The property value, or an empty string if the property was not found.
*
********************************************************************************
*/
function PropertyValue(propName)
{
    var answer = "";

    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    var prop = propMgr.GetProp(propName);
    if (prop)
    {
        answer = prop.value;
    }

    return answer;
}

/**
********************************************************************************
*
* GetNcHome()
*
* Get the value of the NCHOME environment variable.
*
* @return The value of the NCHOME environment variable.
*
********************************************************************************
*/
function GetNcHome()
{
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    return propMgr.GetProp("$NCHOME").value;
}

/**
********************************************************************************
*
* MySQLGrantAdminRights()
*
* Function to grant the administrator rights to the MySQL datbase
*
********************************************************************************
*/
function MySQLGrantAdminRights()
{
    var user = PropertyValue("MYSQL50.ADMINUSERNAME");
    var pwd  = PropertyValue("MYSQL50.ADMINPASSWORD");
    var host = PropertyValue("MYSQL50.HOST");

    var pcnt = "@'%'";
    if (ncipackage.shortArch == "win32")
    {
        pcnt = "@'%%'";
    }

    var grant = "DELETE FROM mysql.user; "
              + "GRANT ALL ON *.* TO " + user + pcnt + " IDENTIFIED BY '" + pwd + "' WITH GRANT OPTION; "
              + "GRANT ALL ON *.* TO " + user + "@localhost IDENTIFIED BY '" + pwd + "' WITH GRANT OPTION; "
              + "GRANT ALL ON *.* TO " + user + "@'" + host + "' IDENTIFIED BY '" + pwd + "' WITH GRANT OPTION; "
              + "FLUSH PRIVILEGES;";

    //
    // Use the default admin user of "root" with no password
    //
    MySQLExecuteGrant("root", "", grant, "all");
}

/**
********************************************************************************
*
* MySQLGrantUserRights(user, pwd, schema)
*
* Function to grant rights to a particular database for a given user
*
********************************************************************************
*/
function MySQLGrantUserRights(user, pwd, schema)
{
    var host      = PropertyValue("MYSQL50.HOST");
    var adminUser = PropertyValue("MYSQL50.ADMINUSERNAME");
    var adminPwd  = PropertyValue("MYSQL50.ADMINPASSWORD");

    //
    // The admin user will already have permission - and we don't what to change the admin password where
    //
    if (user == adminUser)
    {
        return;
    }

    var pcnt = "@'%'";
    if (ncipackage.shortArch == "win32")
    {
        pcnt = "@'%%'";
    }

    var privs = " ALTER,INDEX,INSERT,DELETE,UPDATE,SELECT,CREATE,DROP,EXECUTE,CREATE VIEW, CREATE ROUTINE ";
    var grant = "GRANT" + privs + "ON " + schema + ".* TO " + user + pcnt + " IDENTIFIED BY '" + pwd + "'; "
              + "GRANT" + privs + "ON " + schema + ".* TO " + user + "@localhost IDENTIFIED BY '" + pwd + "'; "
              + "GRANT" + privs + "ON " + schema + ".* TO " + user + "@'" + host + "' IDENTIFIED BY '" + pwd + "'; "
              + "FLUSH PRIVILEGES;";

    //
    // Use the - now configured - MySQL admin username and password
    //
    MySQLExecuteGrant(adminUser, adminPwd, grant, schema);
}

/**
********************************************************************************
*
* MySQLExecuteGrant(user, pwd, grant, schema)
*
* Function to grant the requsted user permissions on the MySQL database.
*
********************************************************************************
*/
function MySQLExecuteGrant(user, pwd, grant, schema)
{
    //
    // Set up platform specific variables
    //
    var separator = "/";
    var topLine = "#!/bin/sh";

    var mysqlCmd = GetNcHome() + "/platform/" + ncipackage.shortArch + "/mysql5.0/bin/mysql";
    var testIf   = "if [ -f \"" + mysqlCmd + "\" ]; then";
    var testFi   = "fi";
    var mysqlEnv = "MYSQL_HOME=\"" + GetNcHome() + "/platform/" + ncipackage.shortArch + "/mysql5.0\"\n"
                 + "export MYSQL_HOME";

    //
    // May need to give MySQL Server time to start
    //
    var sleep = "";
    if (schema == "all")
    {
        sleep = "sleep 10";
    }

    if (ncipackage.shortArch == "win32")
    {
        separator = "\\";
        topLine = "@echo off";
        mysqlCmd = GetNcHome() + "\\platform\\win32\\mysql5.0\\bin\\mysql.exe";
        testIf   = "@if not exist \"" + mysqlCmd + "\" goto end";
        testFi   = ":end";
        mysqlEnv = "";

        if (schema == "all")
        {
            // No sleep command on Windows
            sleep = "ping -n 11 127.0.0.1 > nul";
        }
    }

    //
    // Now attempt to execute a script which will grant the user permissions
    //
    try
    {
        //
        // Now create a script to grant the permissions on the database
        //
        var scriptFileName = GetNcHome() + separator + "mysql_grant_" + schema + ".bat";

        var scriptFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        scriptFile.initWithPath(scriptFileName);

        var pwdStr = "";
        if (pwd)
        {
            pwdStr = " -p" + pwd;
        }

        var scriptContents = topLine + "\n"
                           + testIf + "\n"
                           + mysqlEnv + "\n"
                           + sleep + "\n"
                           + "\"" + mysqlCmd + "\""
                                + " --silent"
                                + " --user " + user
                                + pwdStr
                                + " --port " + PropertyValue("MYSQL50.PORT")
                                + " --wait "
                                + " --connect_timeout=60 "
                                + " --execute \"" + grant + "\"\n"
                           + testFi + "\n";

        var foStream2 = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        foStream2.init(scriptFile, 0x02 | 0x08 | 0x20, 0777, 0); // write, create, truncate
        foStream2.write(scriptContents, scriptContents.length);
        foStream2.close();

        //
        // Now run the script.
        //
        var process = Components.classes["@mozilla.org/process/util;1"].
            createInstance(Components.interfaces.nsIProcess);
        process.init(scriptFile);

        var args = [];
        process.run(true, args, args.length);

        //
        // Finally, clean up the temporary file.
        //
        scriptFile.remove(false);
    }
    catch(e)
    {
        Log(0, e);
    }
}


