/******************************************************************/
/*                                                                */
/* Copyright (C) 1994 - 2005, Micromuse, Ltd.                     */
/* All Rights Reserved                                            */
/*                                                                */
/*----------------------------------------------------------------*/
/*                                                                */
/* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE                    */
/* The copyright notice above does not evidence any               */
/* actual or intended publication of such source code.            */
/*                                                                */
/* RESTRICTED RIGHTS:                                             */
/*                                                                */
/* This file may have been supplied under a license.              */
/* It may be used, disclosed, and/or copied only as permitted     */
/* under such license agreement. Any copy must contain the        */
/* above copyright notice and this restricted rights notice.      */
/* Use, copying, and/or disclosure of the file is strictly        */
/* prohibited unless otherwise provided in the license agreement. */
/*                                                                */
/******************************************************************/

/**
********************************************************************************
*
* CreateCtrlServices()
*
* Function to create a CtrlServices.DOMAIN.cfg file, based on the default
* version but with "NCOMS" substituted for the name of the ObjectServer
* Precision is integrating with.
*
********************************************************************************
*/
function CreateCtrlServices()
{
    var objserv = PropertyValue("OBJECTSERVER0.NAME");

    if (objserv.length > 0)
    {
        var replaceRegex = new RegExp("\"NCOMS\"", "g");
        var configFile = "CtrlServices";

        CreateDomainSpecificConfigFile(configFile, replaceRegex, objserv);
    }
}

/**
********************************************************************************
*
* CreateWebtopDataSourceCfg()
*
* Function to create a ModelNcimDb.DOMAIN.cfg file, based on the default
* version but with "NCOMS" substituted for the name of the webtop datasource,
* but only do so if the user is not using "NCOMS".
*
********************************************************************************
*/
function CreateWebtopDataSourceCfg()
{
    var ncwDataSource = PropertyValue("NCWDATASOURCE0.NAME");

    if ( ncwDataSource.length > 0 && ncwDataSource != "NCOMS")
    {
        var replaceRegex = new RegExp("\"NCOMS\"", "g");
        var configFile = "ModelNcimDb";

        CreateDomainSpecificConfigFile(configFile, replaceRegex, ncwDataSource);
    }
}

/**
********************************************************************************
*
* CreateDomainSpecificConfigFile(configFile, replaceRegex, newStr)
*
* Function to create a domain specific configuration file by copying the 
* default file and then applying a regular expression to modify it.
*
********************************************************************************
*/
function CreateDomainSpecificConfigFile(configFile, replaceRegex, newStr)
{
    var domainName = PropertyValue("PRECISION.DOMAIN0");

    if (domainName.length > 0)
    {
        //
        // Set up platform specific variables
        //
        var separator = "/";
        if (ncipackage.shortArch == "win32")
        {
            separator = "\\";
        }

        var replaceRegex = new RegExp("\"NCOMS\"", "g");

        try
        {
            var newFileName = GetNcHome() + separator + "etc" + separator
                            + "precision" + separator + configFile + "."
                            + domainName + ".cfg";

            var newFile = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            newFile.initWithPath(newFileName);

            //
            // Only create a <configFile>.DOMAIN.cfg if there isn't one already.
            //
            if (newFile.exists())
            {
                Log(1, "Not creating " + newFileName + " because it already exists");
            }
            else
            {
                var defaultFileName = GetNcHome() + separator + "etc" + separator
                                    + "precision" + separator + "default" + separator
                                    + configFile + ".cfg";

                var defaultFile = Components.classes["@mozilla.org/file/local;1"].
                    createInstance(Components.interfaces.nsILocalFile);
                defaultFile.initWithPath(defaultFileName);

                if (defaultFile.exists())
                {
                    var defaultCfgFile = ReadFile(defaultFile);

                    var newCfgFile = defaultCfgFile.replace(replaceRegex,
                                                                      "\"" + newStr + "\"");

                    var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                        createInstance(Components.interfaces.nsIFileOutputStream);
                    foStream.init(newFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate
                    foStream.write(newCfgFile, newCfgFile.length);
                    foStream.close();
                }
                else
                {
                    Log(1, "Not creating " + newFileName + " because the template "
                         + defaultFileName + " doesn't exist");
                }
            }
        }
        catch(e)
        {
            Log(0, e);
        }
    }
    else
    {
        Log(0, "Unable to create " + configFile + ".DOMAIN.cfg as Precision domain "
             + "or object server name are not available.");
    }
}

/**
********************************************************************************
*
* UserIsRoot()
*
* Determine if the user running the installation is root.
*
* @return true if the user running the installation is root; otherwise false.
*
********************************************************************************
*/
function UserIsRoot()
{
    var isRoot = false;

    //
    // Can't use ncipackage.shortArch to determine the architecture
    // here, as it needs to work from both a XUL panel and the package
    // XML.
    //
    // We assume there is only a %windir% environment variable on Windows.
    //
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    var isWin = propMgr.GetProp("$windir").value;
    if (isWin)
    {
        //
        // Windows installs must be done as a user with administrative
        // privileges, so they are the equivalent of root.
        //
        isRoot = true;
    }
    else
    {
        //
        // Run the Unix "id" command to check if this is the root user.
        //
        try
        {
            var scriptFilename = GetNcHome() + "/id.sh";

            var tmpScript = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            tmpScript.initWithPath(scriptFilename);

            var contents = "#!/bin/sh\n"
                + "id > " + GetNcHome() + "/id.tmp\n";

            var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
            foStream.init(tmpScript, 0x02 | 0x08 | 0x20, 0777, 0); // write, create, truncate
            foStream.write(contents, contents.length);
            foStream.close();

            var process = Components.classes["@mozilla.org/process/util;1"].
                createInstance(Components.interfaces.nsIProcess);
            process.init(tmpScript);

            // Run the process.
            // If the first param is true, the calling process will be blocked until
            // the called process terminates.
            // Second and third params are used to pass command-line arguments
            // to the process.
            var args = [];
            process.run(true, args, args.length);

            var outputFilename = GetNcHome() + "/id.tmp";

            var tmpOutput = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            tmpOutput.initWithPath(outputFilename);

            var lines = ReadFile(tmpOutput).split("\n");
            if (lines[0].match(/^uid=0\(/))
            {
                isRoot = true;
            }

            tmpScript.remove(false);
            tmpOutput.remove(false);
        }
        catch(e)
        {
            Log(0, "Cannot determine current user ID");
        }
    }

    return isRoot;
}

